"""AD - Groups

This module contains actions for viewing and managing Azure AD Groups.
"""

from typing import List

from ppa import ui

from ..exceptions import ObjectNotFound
from ..records import AzureClient
from ._ad_api import client, process_groups, process_users
from .records import Group, User


def get_all(azure_client: AzureClient) -> List[Group]:
    """Get all AD groups.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

    ??? success "Output"

        A list of [Group](../inputs_outputs/group) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.ad.groups.get_all:
          load:
            azure_client: azure_secrets
          save: all_groups
        ```
    """
    return process_groups(client(client=azure_client).groups.list())


def get_by_name(azure_client: AzureClient, name: str) -> Group:
    """Get an AD group by name

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __name__: Group name

    ??? success "Output"

        A single [Group](../inputs_outputs/#group) dictionary.

    ??? example
        ```YAML linenums="1"
        - azure.ad.groups.get_by_name:
            name: ppa
          load:
            azure_client: azure_secrets
          save: ppa_group
        ```
    """
    try:
        return process_groups(client(client=azure_client).groups.list(filter="displayName eq '{}'".format(name)))[0]
    except IndexError:
        raise ObjectNotFound("Group not Found: {}".format(name))


def get_users(azure_client: AzureClient, group_id: str) -> List[User]:
    """Get users from an Azure AD group's membership list.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __group_id:__ the ID of the group


    ??? success "Output"

        A list of [User](../inputs_outputs/#user) dictionaries.

    ??? example
        Getting all users in _Example Group_, & saving them as a new variable called _user_list_.
        ```YAML linenums="1"
        - azure.ad.groups.get_users:
          load:
            group_id: selected_group_id
            azure_client: azure_secrets
          save: user_list
        ```
    """
    return process_users(client(client=azure_client).groups.get_group_members(group_id))


def output_table(text: str, group_list: List[Group]) -> None:
    """Display a list of Groups in a table. The table will have the following columns:

    The table will have the following columns:

    - Display Name
    - Object ID

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __group_list:__ a list of [Group](../inputs_outputs/#group) dictionaries to display in the table

    ??? success "Output"
        Nothing is outputted by this action.

    ??? example
        - Show a list of groups in a table

        ```YAML linenums="1"
        - azure.ad.groups.output_table:
            text: All groups for user: Jon Smith
          load:
            group_list: user_groups
        ```
    """
    ui.output_table(
        text,
        header=["Display Name", "Object ID"],
        data=[[user.get(field) or "" for field in ["name", "object_id"]] for user in group_list],
    )
