"""AD - Users

This module contains actions for viewing and managing Azure AD Users.
"""

from typing import List

from ppa import ui

from ._ad_api import client, process_users, process_groups
from .records import User, Group
from ..records import AzureClient


def get_all(azure_client: AzureClient) -> List[User]:
    """Get all AD users.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

    ??? success "Output"

        A list of [User](../inputs_outputs/#user) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.ad.users.get_all:
          load:
            azure_client: azure_secrets
          save: all_users
        ```
    """
    return process_users(client(client=azure_client).users.list())


def get_by_name(azure_client: AzureClient, name: str) -> List[User]:
    """Get an AD user by name

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __name__: User name

    ??? success "Output"

         A list of [User](../inputs_outputs/#user) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.ad.users.get_by_name:
            name: Jon Smith
          load:
            azure_client: azure_secrets
          save: ppa_group
        ```
    """
    return process_users(client(client=azure_client).users.list(filter="displayName eq '{}'".format(name)))


def group_memberships(azure_client: AzureClient, user_id: str) -> List[Group]:
    """Get an all group memberships for an AD user

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __user_id__: User Object ID

    ??? success "Output"

         A list of [Group](../inputs_outputs/#group) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.ad.users.group_memberships:
          load:
            user_id: user_id
            azure_client: azure_secrets
          save: user_groups
        ```
    """
    return process_groups(
        client(client=azure_client).groups.list(
            filter="objectID eq '{}'".format(
                "' or objectID eq '".join(
                    [group for group in client(client=azure_client).users.get_member_groups(user_id, False)]
                )
            )
        )
    )


def output_table(text: str, user_list: List[User]) -> None:
    """Display a list of users in a table. The table will have the following columns:

    The table will have the following columns:

    - Display Name
    - Principal Name
    - Object ID

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __user_list:__ a list of [User](../inputs_outputs/#user) dictionaries to display in the table

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example
        - Show a list of users in a table

        ```YAML linenums="1"
        - azure.ad.users.output_table:
            text: All users in AD group: ppa
          load:
            user_list: group_users
        ```
    """
    ui.output_table(
        text,
        header=["Display Name", "Principal Name", "Object ID"],
        data=[[user.get(field) or "" for field in ["name", "principal_name", "object_id"]] for user in user_list],
    )
