"""Compute - Virtual Machines

This module contains actions for viewing and managing Azure Virtual Machines.
"""
import uuid

from typing import List, Optional, Union

from ppa import ui

from ._compute_api import client, process_virtual_machines
from .records import VirtualMachine
from ..records import AzureClient

INSTANCE_TABLE_HEADER = ["Name", "ID"]

INSTANCE_TABLE_FIELDS = [
    lambda instance: instance.get("name"),
    lambda instance: instance.get("id"),
]


def get_all(azure_client: AzureClient, subscription_id: str, resource_group_name: Optional[str] = None) -> List[VirtualMachine]:
    """Get all virtual machines.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __subscription_id__: the subscription ID

        - __resource_group_name:__ the resource group name (Optional)

    ??? success "Output"

        A list of [VirtualMachine](../inputs_outputs/#virtualmachine) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.compute.virtual_machines.get_all:
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: all_virtual_machines
        ```
    """
    return process_virtual_machines(
        client(client=azure_client, subscription_id=subscription_id).virtual_machines.list(resource_group_name)
        if resource_group_name is not None
        else client(client=azure_client, subscription_id=subscription_id).virtual_machines.list_all()
    )


def get_by_name(
    azure_client: AzureClient, subscription_id: str, name: str, resource_group_name: Optional[str] = None
) -> VirtualMachine:
    """Get all virtual machines.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __subscription_id__: the subscription ID

        - __name:__ the virtual machine name

        - __resource_group_name:__ the resource group name (Optional)

    ??? success "Output"

        A Single [VirtualMachine](../inputs_outputs/#virtualmachine) dictionary.

    ??? example
        ```YAML linenums="1"
        - azure.compute.virtual_machines.get_by_name:
            name: ppa-dev
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: ppa_dev_instance
        ```
    """

    return [instance for instance in get_all(azure_client, subscription_id, resource_group_name) if instance["name"] == name]


def get_untagged(
    azure_client: AzureClient, subscription_id: str, resource_group_name: Optional[str] = None
) -> List[VirtualMachine]:
    """Get untagged virtual machines.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __subscription_id__: the subscription ID

        - __resource_group_name:__ the resource group name (Optional)

    ??? success "Output"

        A list of [VirtualMachine](../inputs_outputs/#virtualmachine) dictionaries.

    ??? example
        ```YAML linenums="1"
        - azure.compute.virtual_machines.get_untagged:
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: untagged_virtual_machines
        ```
    """
    return [instance for instance in get_all(azure_client, subscription_id, resource_group_name) if not instance["tags"]]


def stop(azure_client: AzureClient, subscription_id: str, name: str, resource_group_name: str) -> None:
    """Stop an instance.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __subscription_id__: the subscription ID

        - __name:__ the virtual machine name

        - __resource_group_name:__ the resource group name

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example

        ```YAML linenums="1"
        - azure.compute.virtual_machines.stop:
            name: ppa-dev
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
        ```
    """

    async_vm_stop = client(client=azure_client, subscription_id=subscription_id).virtual_machines.power_off(
        resource_group_name, name
    )
    async_vm_stop.wait()


def start(azure_client: AzureClient, subscription_id: str, name: str, resource_group_name: str) -> None:
    """Start an instance.

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __azure_client:__ an [AzureClient](../inputs_outputs/#azureclient) dictionary

        - __subscription_id__: the subscription ID

        - __name:__ the virtual machine name

        - __resource_group_name:__ the resource group name

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example

        ```YAML linenums="1"
        - azure.compute.virtual_machines.start:
            name: ppa-dev
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
        ```
    """

    async_vm_start = client(client=azure_client, subscription_id=subscription_id).virtual_machines.start(
        resource_group_name, name
    )
    async_vm_start.wait()


def output_table(text: str, instances: Union[VirtualMachine, List[VirtualMachine]]) -> None:
    """Display a list of instances in a table.

    The table will have the following columns:

    - Resource Name
    - Resource ID

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __groups:__ a single or list of [VirtualMachine](../inputs_outputs/#virtualmachine) dictionaries

    ??? success "Output"

        Nothing is outputted by this action.

    ??? example "Single Instance"

        Finding an instance with [get_by_name](#get_by_name), saving it as `vm`, & displaying it:

        ```YAML linenums="1" hl_lines="6 11"
        - azure.compute.virtual_machines.get_by_name:
            resource_group_name: ppa
            name: ppa-dev
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: vm

        - azure.compute.virtual_machines.output_table:
            text: ppa-dev
          load:
            instances: vm
        ```

    ??? example "Multiple Instances"

        Finding all instances with [get_all](#get_all), saving them as `instances`, & displaying them:

        ```YAML linenums="1" hl_lines="5 10"
        - azure.compute.virtual_machines.get_all:
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: instances

        - azure.compute.virtual_machines.output_table:
            text: All Instances
          load:
            instances: instances
        ```
    """
    if not isinstance(instances, list):
        instances = [instances]

    ui.output_table(
        text, header=INSTANCE_TABLE_HEADER, data=[[get(instance) for get in INSTANCE_TABLE_FIELDS] for instance in instances]
    )


def input_table(
    text: str, instances: List[VirtualMachine], minimum: Optional[int] = None, maximum: Optional[int] = None
) -> List[VirtualMachine]:
    """Display a list of instances in a table, & allow the task operator to make a selection.

    The table will have the following columns:

    - Resource Name
    - Resource ID

    !!! question "Minimum Plugin Version: 1.0.0"

    ??? info "Input"

        - __text:__ the title of the table

        - __instances:__ a list of [VirtualMachine](../inputs_outputs/#virtualmachine) dictionaries to display in the table

        - __minimum:__ The minimum number of acceptable selections (Optional)

        - __maximum:__ The maximum number of acceptable selections (Optional)

    ??? success "Output"

        A list of [VirtualMachine](../inputs_outputs/#virtualmachine) dictionaries.

    ??? tip

        - If __neither__ a `minimum` _or_ `maximum` is provided, the task operator will be able to submit 0 selections.

        - If `minimum` _or_ `maximum` are provided, the operation will repeat until a valid number of selections is made.

    ??? example

        Finding instances with [get_all](#get_all), saving them as `groups`, & waiting for a single selection:

        ```YAML linenums="1" hl_lines="5 12 13"
        - azure.compute.virtual_machines.get_all:
            resource_group_name: ppa
          load:
            subscription_id: subscription_id
            azure_client: azure_secrets
          save: instances

        - azure.compute.virtual_machines.input_table:
            text: Choose a virtual machine
            minimum: 1
            maximum: 1
          load:
            instances: instances
          save: selected_instance
        ```
    """
    table_element_id = str(uuid.uuid4())
    error_element_id = str(uuid.uuid4())
    while True:
        indices = ui.input_table(
            text,
            header=INSTANCE_TABLE_HEADER,
            data=[[get(instance) for get in INSTANCE_TABLE_FIELDS] for instance in instances],
            element_id=table_element_id,
        )
        count = len(indices)
        if minimum and count < minimum:
            ui.output_error(f"Please choose at least {minimum} record(s)", element_id=error_element_id)
            continue
        if maximum and count > maximum:
            ui.output_error(f"Please choose no more than {maximum} record(s)", element_id=error_element_id)
            continue
        break

    return [instances[i] for i in indices]
